/****************************************************************************
    
    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef NEWTONATORINSTR_H_INCLUDED
#define NEWTONATORINSTR_H_INCLUDED

#include "Newtonator.h"
#include <stk/Instrmnt.h>
#include <list>

class NewtonatorInstr : public stk::Instrmnt
{
    public:
    NewtonatorInstr(unsigned int channels = 2);
    virtual ~NewtonatorInstr();

    //Configuration functions
    void setAllAmpEnvTimes(stk::StkFloat aTime, stk::StkFloat dTime, stk::StkFloat sLevel, stk::StkFloat rTime);
    void setAllGModEnvTimes(stk::StkFloat aTime, stk::StkFloat dTime, stk::StkFloat sLevel, stk::StkFloat rTime);
    void setGravityScale(stk::StkFloat val);
    void setGravity(stk::StkFloat val);
    void setFloorAttraction(stk::StkFloat val);
    void setFloorScale(stk::StkFloat val);
    void setClip(bool val);
    void setGravModDepth(stk::StkFloat val);
    void setFloorType(VariSource::Type type);
    void setGravModType(VariSource::Type type);

    //Overridden virtual functions from Instrmnt class
    stk::StkFloat tick(unsigned int channel = 0);
    void noteOn(stk::StkFloat frequency, stk::StkFloat amplitude);
    void noteOff(stk::StkFloat amplitude);
    void setFrequency(stk::StkFloat frequency);
    void controlChange(int number, stk::StkFloat value);

    protected:
    void sampleRateChanged(stk::StkFloat newRate, stk::StkFloat oldRate);
    virtual void initChannels(unsigned int numChannels);
    void noteOff(stk::StkFloat frequency, stk::StkFloat amplitude);
    void allOff();
    void addActiveNewt(Newtonator* newt);
    Newtonator* getNextAvailNewt();

    typedef std::list<Newtonator*> NewtCollection;
    typedef std::list<Newtonator*> PolyphonyQueue;
    NewtCollection _newtonators;
    PolyphonyQueue _activeNewts;
    NewtCollection::iterator _currNewtIt;
    static const int NUM_NOTES;
    static const size_t POLYPHONY;

    Newtonator* _currNewt;
};

#endif // NEWTONATORINSTR_H_INCLUDED
